﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HeuristicLab.DataPreprocessing.Views {
  class FindPreprocessingItemsIterator : IFindPreprocessingItemsIterator {
    private IDictionary<int, IList<int>> items;
    private Tuple<int, int> currentCell;

    public FindPreprocessingItemsIterator(IDictionary<int, IList<int>> items) {
      this.items = items;
      Reset();
    }

    public bool MoveNext() {
      bool result = false;
      bool endReached = false;
      if (items != null) {
        do {
          if (currentCell.Item2 < items[currentCell.Item1].Count - 1) {
            currentCell = new Tuple<int, int>(currentCell.Item1, currentCell.Item2 + 1);
          } else if (currentCell.Item1 < items.Count - 1) {
            currentCell = new Tuple<int, int>(currentCell.Item1 + 1, 0);
          } else {
            endReached = true;
          }
        } while (!endReached && !CurrentCellExists());
        result = !endReached;
      }
      return result;
    }

    public Tuple<int, int> GetCurrent() {
      Tuple<int, int> resultCells = null;
      if (items != null && CurrentCellExists()) {
        int cellRow = items[currentCell.Item1].ElementAt(currentCell.Item2);
        resultCells = new Tuple<int, int>(currentCell.Item1, cellRow);
      }
      return resultCells;
    }

    public void Reset() {
      currentCell = new Tuple<int, int>(0, 0);
      if (!CurrentCellExists()) {
        MoveNext();
      }
    }

    private bool CurrentCellExists() {
      bool result = false;
      if (items != null) {
        result = items.ContainsKey(currentCell.Item1) && currentCell.Item2 < items[currentCell.Item1].Count;
      }
      return result;
    }
  }
}
